/*
  Program fftw_example:
  this program computes the Fourier's coefficients of a sawtooth function
  and reconstructs the original function with Nmax harmonics.
  It uses the fftw library to compute the FFTs!
*/

#include <iostream>
#include <cmath>
#include <fstream>
#include <fftw3.h>

using namespace std;

int main()
{
  double *in, *tt, *ff;
  fftw_complex *out;
  fftw_plan pf, pb;

  // Variable allocation

  int n = 128;            // Total number of intervals
  int n2 = n / 2 + 1;     // Nyquist number + 1
  tt = ( double * ) fftw_malloc( sizeof( double ) * n );
  ff = ( double * ) fftw_malloc( sizeof( double ) * n );
  in = ( double * ) fftw_malloc( sizeof( double ) * n );
  out = ( fftw_complex * ) fftw_malloc( sizeof( fftw_complex ) * n2 );

  // Create the forward and backward transform planes
  // Notice the swap of in and out in the backward plan!

  unsigned int flags = FFTW_ESTIMATE || FFTW_DESTROY_INPUT;
  pf = fftw_plan_dft_r2c_1d( n, in, out, flags );
  pb = fftw_plan_dft_c2r_1d( n, out, in, flags );

  // Define the input vector as the sawtooth function

  double pi = acos( -1.0 );
  double T  = 2.0 * pi;
  double dt = T / n;
  for ( int i = 0; i < n; i++ )
    {
      double t = i * dt;
      tt[i] = t;
      if ( t < pi ) ff[i] = t;
      else if ( t == pi ) ff[i] = 0.0;
      else ff[i] = t - 2.0 * pi;
    }
  
  // Computes the forward transform
  
  for ( int i = 0; i < n; i++ ) in[i] = ff[i];
  fftw_execute( pf );

  // Normalize the transform!

  for ( int i = 0; i <= n2; i++ )
    {
      out[i][0] = out[i][0] / n;     // Normalize the real parts of the output vector
      out[i][1] = out[i][1] / n;     // Normalize the imaginary parts of the output vector
    }

  cout << "Please, input Nmax (smaller than " << n2 - 1 << "): " << endl;
  int Nmax;
  cin >> Nmax;

  // Fills up with zeros the harmonics greater than Nmax

  for ( int i = Nmax; i <= n2; i++ )
    {
      out[i][0] = 0.0;
      out[i][1] = 0.0;
    }
  
  // Computes the backward transform
  
  fftw_execute( pb );

  // Write the vectors to a file

  ofstream fout;
  fout.open( "funct_rec.dat" );
  fout.precision( 15 );
  for ( int i = 0; i < n; i++ )
    {
      fout << tt[i] << "\t" << ff[i] << "\t" << in[i] << endl;
    }
  fout.close();
  
  // Variable and plan de-allocation

  fftw_destroy_plan( pf );
  fftw_destroy_plan( pb );
  fftw_free( in );
  fftw_free( out );
  return 0;
}
